#******************************************************************************
#
#       Copyright:      2006-2009 Paul Obermeier (obermeier@tcl3d.org)
#
#                       See the file "Tcl3D_License.txt" for information on
#                       usage and redistribution of this file, and for a
#                       DISCLAIMER OF ALL WARRANTIES.
#
#       Module:         Tcl3D -> starpack
#       Filename:       tcl3dsh.tcl
#
#       Author:         Paul Obermeier
#
#       Description:    Startup code for the Tcl3D Starpack tcl3dsh.
#
#******************************************************************************

# Initialize Tk and the sub-package tcl3dUtil for displaying the logo 
# splash screen.
package require Tk
package require tcl3dogl

tcl3dLogoShowPoSoft "Tcl3D Version 0.4.1" \
        "Copyright 2005-2009 Paul Obermeier" .
tcl3dLogoDestroyPoSoft

# Check if all necessary external libraries exists in the directory 
# containing the Starpack. Copy them to the filesystem, if necessary.
set __tcl3dExecDir [file dirname $::starkit::topdir]
set __tcl3dDllList [glob -nocomplain -dir [file join $starkit::topdir extlibs] \
                  *[info sharedlibextension]*]

foreach starkitName $__tcl3dDllList {
    set osName [file join $__tcl3dExecDir [file tail $starkitName]]
        if { ! [file exists $osName] } {
        set retVal [catch { file copy -force -- $starkitName $__tcl3dExecDir }]
        # puts "Copying DLL $starkitName to directory $__tcl3dExecDir"
        if { $retVal != 0 } {
            error "Error copying DLL $starkitName to directory $__tcl3dExecDir"
        }
    } else {
        # puts "No need to copy"
    }
}

package require tcl3d

proc writeTcl3dDemos { dirName } {
    if { ! [file isdirectory $dirName] } {
        file mkdir $dirName
    }
    set starkitName [file join $starkit::topdir "demos"]
    set retVal [catch { file copy -force -- $starkitName $dirName }]
    if { $retVal != 0 } {
        error "Error copying Tcl3D demos $starkitName to directory $dirName"
    }
}

proc writeTcl3dPkgs { dirName } {
    if { ! [file isdirectory $dirName] } {
        file mkdir $dirName
    }
    set tcl3dPkgList [glob -nocomplain -dir [file join $starkit::topdir lib] *]
    foreach starkitName $tcl3dPkgList {
        if { [string match "tcl3d*" [file tail $starkitName]] } {
            set retVal [catch { file copy -force -- $starkitName $dirName }]
            if { $retVal != 0 } {
                error "Error copying Tcl3D package $starkitName to directory $dirName"
            }
        }
    }
}

proc writeTcl3dExtLibs { dirName } {
    if { ! [file isdirectory $dirName] } {
        file mkdir $dirName
    }
    set starkitName [file join $starkit::topdir "extlibs"]
    set retVal [catch { file copy -force -- $starkitName $dirName }]
    if { $retVal != 0 } {
        error "Error copying external libraries $starkitName to directory $dirName"
    }
}

# Command to start the built-in presentation.
proc pres {} {
    set name [file join $starkit::topdir "demos" "presentation.tcl"]
    uplevel #0 source [list $name]
}

# Command to write the installation packages to disk.
proc inst {} {
    set name [file dirname $starkit::topdir]
    writeTcl3dDemos   $name
    writeTcl3dPkgs    $name
    writeTcl3dExtLibs $name
    puts "Installation packages written to $name"
}

wm title . "Tcl3D"

# If no parameters have been supplied, open a console window and
# give the user a hint about starting the presentation.
if { $argc == 0 } {
    tcl3dConsoleCreate .tcl3dConsole "tcl3d> " "Tcl3D Console"
    puts "Type \"pres\" to start Tcl3D presentation."
    puts "Type \"inst\" to write the Tcl3D installation packages to disk."
    puts ""
    puts "Note:"
    puts "  The OpenSceneGraph library and demos are not included in this presentation."
    if { $::tcl_platform(os) eq "Darwin" || $::tcl_platform(os) eq "IRIX64" } {
        puts "  The Cg library is not included in this presentation."
    }
    update
} else {
    set i 0
    while { $i < $argc } {
        if { [lindex $argv $i] eq "-console" } {
            # Start a console window. This parameter must be specified before
            # the -pres parameter and before a Tcl script parameter.
            tcl3dConsoleCreate .tcl3dConsole "tcl3d> " "Tcl3D Console"
            incr argc -1
            set argv [lrange $argv [expr $i+1] end]
        } elseif { [lindex $argv $i] eq "-pres" } {
            # If option -pres is given, start presentation immediately.
            # Make sure, no command line parameters are given to the
            # presentation framework, as they may disturb the default
            # settings for some of the demo programs.
            set argc 0
            set argv [list]
            pres
            break
        } elseif { [lindex $argv $i] eq "-inst" } {
            # If option -inst is given, write out the installation packages
            # and quit.
            inst
            exit
        } else {
            # If other command line parameters are supplied, assume the first
            # is the name of a Tcl script, which will be sourced.
            # Decrement the command argument counter and remove the sourced file
            # name from the command line parameter list.
            set tclScript [file normalize [lindex $argv $i]]
            incr argc -1
            set argv [lrange $argv [expr $i+1] end]
            set argv0 [vfs::filesystem fullynormalize $tclScript]
            source $tclScript
            break
        }
    }
}
